#ifndef UHU_MENU_H
#define UHU_MENU_H

#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include <langinfo.h>
#include <iconv.h>
#include <dlfcn.h>

typedef int bool;

struct charset {
	char *name;
	iconv_t from_utf8;
	iconv_t to_utf8;
	struct charset *next;
};

struct language {
	char *name;
	struct charset *charset;
	locale_t locale;
	locale_t locale_utf8;
	struct language *next;
};

struct lang {
	struct language *language;
	char *value;
	char *value_8bit;
	struct lang *next;
};

struct key {
	char *name;
	struct lang *langs;
	struct lang *current_lang;
	struct key *next;
};

struct section {
	char *name;
	struct key *keys;
	struct section *next;
};

struct item {
	bool issubmenu;
	struct menu *submenu;
	char *file_name;
	struct section *sections;
	struct key *key_name;
	struct key *key_major;
	struct key *key_minor;
	char *onlyshowin;
	char *notshowin;
	int major;
	int minor;
	bool terminal;
#ifdef UHU_MENU_CORE_ENGINE
	bool valid_general;
	bool valid_lang;
	bool valid_wm;
	bool valid;
#else
	bool do_not_use_me_valid_general;
	bool do_not_use_me_valid_lang;
	bool do_not_use_me_valid_wm;
	bool do_not_use_me_valid;
#endif
};

struct menu {
	int nr;
	struct item *items;
	int
#ifdef UHU_MENU_CORE_ENGINE
	    nr_real;
#else
	    do_not_use_me;
#endif
};

extern struct charset *first_charset;
extern struct language *first_language;

extern int plugin_type;
extern const char *plugin_name;

extern int loglevel;

struct menu *read_menu (const char *path);
int write_menu (const struct menu *menu, const char *path);
int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language);

locale_t newlocale2 (const char *locale, int utf8);
int convert (iconv_t ic, /* XXX const */ char *from, char **to);

struct charset *find_or_create_charset (const char *name);
struct language *find_or_create_language (const char *name);

int section_add_value (struct section *section, const char *key,
  const char *lang, const char *value);
struct section *item_add_section (struct item *item, const char *section);
struct lang *key_find_lang (const struct key *key, const char *lang);
struct key *section_find_key (const struct section *section, const char *key);
struct section *item_find_section (const struct item *item, const char *section);
struct key *item_find_key (const struct item *item, const char *section,
  const char *key);
struct key *item_find_key (const struct item *item, const char *section,
  const char *key);
char *get_value (const struct item *item, const char *section,
  const char *key, const char *lang);
char *get_value_8bit (const struct item *item, const char *section,
  const char *key, const char *lang);
int get_value_bool (const struct item *item, const char *section,
  const char *key, const char *lang);

/*
 *  Recursively remove contents of a directory (like "rm -rf" does it),
 *  except this directory itself (unlike "rm -rf").
 */
int empty_dir (const char *path);

/*
 *  A small wrapper around sscanf: returns the integer value of the string,
 *  or the value "err" in case of error.
 */
int string_to_int (const char *s, int err);

/*
 *  Test if word appears in list, where items are separated by any character in delim.
 *  Case insensitive. Returns true (1) or false (0).
 */
int is_in_list (const char *word, const char *list, const char *delim);

/*
 *  Print a message to stderr. level goes from 0 (critical) to 7 (debug)
 *  just like for syslog. format is a printf-style format without a
 *  trailing newline. message_e() also prints strerror(errno).
 */
void message(int level, const char *format, ...);
void message_e(int level, const char *format, ...);



#endif
